﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public enum ElevatorActionEnum{
	up,
	down,
	pendularVertical,
    pendularHorizontal
}

public abstract class ElevatorClass : ActivatableClass, IForceEffect {

	//protected ElevatorSoundBehaviour elevatorSoundScr;
    [TooltipAttribute("When the elevator is on pendular " +
    	"vertical this should be the most elevated point" +
    	"/" +
    	"When the elevator is on pendular horizontal this should" +
    	"be the point to the left")]
	[SerializeField] protected Transform startPoint;
    [TooltipAttribute("When the elevator is on pendular " +
        "vertical this should be the least elevated point" +
        "/" +
        "When the elevator is on pendular horizontal this should" +
        "be the point to the right")]
    [SerializeField] protected Transform endPoint;
	protected float startValue;
	protected float endValue;
	[SerializeField] protected ElevatorActionEnum actionOnCharge;
	[Range(0,5)][SerializeField] protected float moveSpeed;
	protected float speedMultiplier;
	[SerializeField] protected bool active;
    [SerializeField] protected bool penduleState = true;
    protected SurfaceEffector2D surfaceEff = null;
    protected Vector3 curForce = new Vector3();

	protected override void SetReferences(){
		//elevatorSoundScr = GetComponent<ElevatorSoundBehaviour> ();

		if (actionOnCharge == ElevatorActionEnum.up || actionOnCharge == ElevatorActionEnum.down || actionOnCharge == ElevatorActionEnum.pendularVertical) {
			startValue = startPoint.position.y;
			endValue = endPoint.position.y;
        }
        else
        {
            if(actionOnCharge == ElevatorActionEnum.pendularHorizontal)
            {
                startValue = startPoint.position.x;
                endValue = endPoint.position.x;
          //      surfaceEff = GetComponent<SurfaceEffector2D>();
          //      surfaceEff.forceScale = 1;
            }
        }
    }

	protected virtual void Start(){
		SetReferences ();
	}

	protected virtual void FixedUpdate(){
		Move ();
	}
		
	protected override void Activate ()
	{
		base.Activate ();
		active = true;
	}

	protected override void Deactivate ()
	{
		active = false;
	}

	protected void Move(){
		if (active) {
			speedMultiplier = 1;
            //rollBackTimer = rollBackTimerMax;
            switch (actionOnCharge)
            {
                case (ElevatorActionEnum.up):
                    if (transform.position.y < endValue)
                    {

                        transform.Translate(Vector2.up * moveSpeed * speedMultiplier * Time.deltaTime);
                        //elevatorSoundScr.PlayElevatorSound ();

                        if (transform.position.y > endValue)
                        {
                            Vector3 pos = transform.position;
                            pos.y = endValue;
                            transform.position = pos;
                        }
                    }
                    else
                    {
                        //elevatorSoundScr.StopElevatorSounds ();
                    }
                    if (transform.position.y == endValue)
                    {
                        fullyActive = true;
                    }
                    break;

                case (ElevatorActionEnum.down):
                    if (transform.position.y > endValue)
                    {

                        transform.Translate(Vector2.down * moveSpeed * speedMultiplier * Time.deltaTime);
                        //elevatorSoundScr.PlayElevatorSound ();

                        if (transform.position.y < endValue)
                        {
                            Vector3 pos = transform.position;
                            pos.y = endValue;
                            transform.position = pos;
                        }
                    }
                    else
                    {
                        //elevatorSoundScr.StopElevatorSounds ();
                    }
                    if (transform.position.y == endValue)
                    {
                        fullyActive = true;
                    }
                    break;
                case (ElevatorActionEnum.pendularVertical):
                    if (penduleState)
                    {
                        Translate(Vector3.down);

                        if (transform.position.y < endValue)
                        {
                            Vector3 pos = transform.position;
                            pos.y = endValue;
                            transform.position = pos;
                            penduleState = false;
                        }


                    }
                    else {
                        Translate(Vector3.up);

                        if (transform.position.y > startValue)
                        {
                            Vector3 pos = transform.position;
                            pos.y = startValue;
                            transform.position = pos;
                            penduleState = true;
                        }

                    }
                    break;
                case (ElevatorActionEnum.pendularHorizontal):
                    float signal = 0;
                    if (penduleState)
                    {
                        Translate(Vector3.right);
                        signal = 1;

                        if (transform.position.x > endValue) {
                            Vector3 pos = transform.position;
                            pos.x = endValue;
                            transform.position = pos;
                            penduleState = false;

                        }

                    }
                    else
                    {
                        Translate(Vector3.left);
                        signal = -1;

                        if (transform.position.x < startValue)
                        {
                            Vector3 pos = transform.position;
                            pos.x = startValue;
                            transform.position = pos;
                            penduleState = true;

                        }

                    }
                    
                    //surfaceEff.speed = (moveSpeed* signal);
                    

                    break;

                default:
                    Debug.LogWarning("Movement mode not implemented");
                    break;

            }


        } else {
			fullyActive = false;
			speedMultiplier = 0.5f;

		    if (actionOnCharge == ElevatorActionEnum.up) {
				if (transform.position.y > startValue) {
				
				transform.Translate (Vector2.down * moveSpeed * speedMultiplier * Time.deltaTime);
				//elevatorSoundScr.PlayElevatorSound ();
                    if (transform.position.y < startValue) {
						Vector3 pos = transform.position;
						pos.y = startValue;
						transform.position = pos;
					}
				}else {
					//elevatorSoundScr.StopElevatorSounds ();
				}
			} else if (actionOnCharge == ElevatorActionEnum.down) {
				if (transform.position.y < startValue) {
				
				transform.Translate (Vector2.up * moveSpeed * speedMultiplier * Time.deltaTime);
				//elevatorSoundScr.PlayElevatorSound ();

					if (transform.position.y > startValue) {
						Vector3 pos = transform.position;
						pos.y = startValue;
						transform.position = pos;
					}
			}else {
				//elevatorSoundScr.StopElevatorSounds ();
			}
           }
		}	
	}

    protected void Translate(Vector3 dir) {
        curForce = dir * moveSpeed * speedMultiplier;
        transform.Translate(curForce * Time.deltaTime);
    }

    public Vector3 GetCurForce()
    {
        return curForce;
    }
}
