﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System;

public static class EventManager{

	static event Action gameOverEvent;

	public static void CallGameOver ()
	{
		var handler = gameOverEvent;
		if (handler != null)
			handler ();
	}

	public static void SubscribeGameOver(Action method){
		gameOverEvent += method;
	}

	public static void UnSubscribeGameOver(Action method){
		gameOverEvent -= method;
	}

	static event Action winGameEvent;

	public static void CallWinGame ()
	{
		var handler = winGameEvent;
		if (handler != null)
			handler ();
	}

	public static void SubscribeWinGame(Action method){
		winGameEvent += method;
	}

	public static void UnSubscribeWingale(Action method){
		winGameEvent -= method;
	}

}
	
public class TooglerClass {
	 
	protected static bool active;

	public static bool Active {
		get {
			return active;
		}
	}
		
	public delegate void Toogler();

	protected static void  CallEvent (Toogler calledEvent, ToogleState state)
	{
		if (calledEvent != null) {
			switch(state){

			case ToogleState.active:
				active = true;
				break;
			case ToogleState.inactive:
				active = false;
				break;
			default:
				Debug.LogWarning("Toogler state type not implemented");
				break;

			}
			calledEvent ();
		} else {
			Debug.LogWarning (state.ToString() + " event is attempting to be called while empty");
		}
	}

	public enum ToogleState
	{
		active,
		inactive
	}

	protected static void ToogleEvents(Toogler activeEvent, Toogler inativeEvent){
		if (active) {
			CallEvent (inativeEvent, ToogleState.inactive);
		} else {
			CallEvent (activeEvent, ToogleState.active);
		}
	}
}

public class PauseEventClass : TooglerClass{

	/// <summary>
	/// Occurs when game is pause(active).
	/// </summary>
	public static event Toogler PauseEvent;
	/// <summary>
	/// Occurs when game is unpaused(inactive).
	/// </summary>
	public static event Toogler UnPauseEvent;

	/// <summary>
	/// Toogles between active and inactive events.
	/// </summary>
	public static void CallToogle(){
		ToogleEvents (PauseEvent,UnPauseEvent);
	}

	public static void CallPause(){
        if (PauseEvent != null)
        {
            PauseEvent();
            active = true;
        }
        else
        {
            Debug.LogError("Pause Event is null");
        }
    }

	public static void CallUnPause(){
		UnPauseEvent();
		active = false;
	}
}
