﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.SceneManagement;

public class UiManager : MonoBehaviour {

	[Header("GameOver")]
	[SerializeField] GameObject gameOverUi =  null;

    [Header("WinGame")]
    [SerializeField] GameObject winGameUi = null;

	void Awake(){
		EventManager.SubscribeGameOver (OnGameOver);
		EventManager.SubscribeWinGame (OnGameWin);
	}
	
	void OnGameOver(){
		if(gameOverUi !=  null)
		gameOverUi.SetActive (true);
	}
		
	void OnGameWin(){
		if(winGameUi != null)
		winGameUi.SetActive (true);
        PauseEventClass.CallPause();
	
	}

	public void MainMenu(){
		SceneManager.LoadScene("MainMenu");
	}

    public void Restart()
    {
        Time.timeScale = 1;
        SceneManager.LoadScene("GamePlay");
    }

    enum FadeTypeEnum
	{
		fadeIn,
		fadeOut
	}

	enum TimeTypeEnum{
		noTime,
		scaled,
		unscaled
	}

	IEnumerator Fade(FadeTypeEnum fadeType, RawImage fadeImage, float fadeSpeed, float endValue, Action OnComplete,TimeTypeEnum timeType = TimeTypeEnum.scaled){

		if (fadeImage != null && fadeSpeed > 0) {

			Color newColor = fadeImage.color;

			switch (fadeType) {
			case FadeTypeEnum.fadeIn:
				switch (timeType) {
				case TimeTypeEnum.noTime:
					while (fadeImage.color.a < endValue) {
						newColor.a += fadeSpeed;
						fadeImage.color = newColor;
						yield return null;
					}
					break;
				case TimeTypeEnum.scaled:
					while (fadeImage.color.a < endValue) {
						newColor.a += fadeSpeed * Time.deltaTime;
						fadeImage.color = newColor;
						yield return null;
					}
					break;
				case TimeTypeEnum.unscaled:
					while (fadeImage.color.a < endValue) {
						newColor.a += fadeSpeed * Time.unscaledDeltaTime;
						fadeImage.color = newColor;
						yield return null;
					}
					break;
				}

				break;
			case FadeTypeEnum.fadeOut:
				switch (timeType) {
				case TimeTypeEnum.noTime:
					while (fadeImage.color.a > endValue) {
						newColor.a -= fadeSpeed;
						fadeImage.color = newColor;
						yield return null;
					}
					break;
				case TimeTypeEnum.scaled:
					while (fadeImage.color.a > 0) {
						newColor.a -= fadeSpeed * Time.deltaTime;
						fadeImage.color = newColor;
						yield return null;
					}
					break;
				case TimeTypeEnum.unscaled:
					while (fadeImage.color.a > endValue) {
						newColor.a -= fadeSpeed * Time.unscaledDeltaTime;
						fadeImage.color = newColor;
						yield return null;
					}
					break;
				}
				break;
			}
			OnComplete ();
		} else {
			Debug.LogError ("Fade Image is null or fade speed is equal or lower than zero: fadeSpeed:" + fadeSpeed);
		}
	}

}
